==> open-xchange-documents-backend_7.8.4.orig.tar.bz2: open-xchange-documents-backend-7.8.4/com.openexchange.office.filter.odf/src/com/openexchange/office/filter/odf/styles/StyleTableCell.java <==
/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2016 OX Software GmbH
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

/**
 * @author sven.jacobi@open-xchange.com
 */

package com.openexchange.office.filter.odf.styles;

import java.util.Map;
import org.apache.xml.serializer.SerializationHandler;
import org.json.JSONException;
import org.json.JSONObject;
import org.xml.sax.SAXException;
import com.openexchange.office.filter.odf.AttributeImpl;
import com.openexchange.office.filter.odf.AttributesImpl;
import com.openexchange.office.filter.odf.Namespaces;
import com.openexchange.office.filter.odf.OpAttrs;
import com.openexchange.office.filter.odf.SaxContextHandler;
import com.openexchange.office.filter.odf.properties.LoExtTableCellGraphicProperties;
import com.openexchange.office.filter.odf.properties.ParagraphProperties;
import com.openexchange.office.filter.odf.properties.TableCellProperties;
import com.openexchange.office.filter.odf.properties.TextProperties;

final public class StyleTableCell extends StyleBase implements ITextProperties, IParagraphProperties, IGraphicProperties {

    private TableCellProperties             tableCellProperties = new TableCellProperties(new AttributesImpl());
    private ParagraphProperties             paragraphProperties = new ParagraphProperties(new AttributesImpl());
    private TextProperties                  textProperties      = new TextProperties(new AttributesImpl());
    private LoExtTableCellGraphicProperties loExtGraphicProperties;

    public StyleTableCell(String name, boolean automaticStyle, boolean contentStyle) {
        super("table-cell", name, automaticStyle, contentStyle);

        loExtGraphicProperties = new LoExtTableCellGraphicProperties(new AttributesImpl(), this, false);
    }

    public StyleTableCell(String name, AttributesImpl attributesImpl, boolean defaultStyle, boolean automaticStyle, boolean contentStyle) {
        super(name, attributesImpl, defaultStyle, automaticStyle, contentStyle);

        loExtGraphicProperties = new LoExtTableCellGraphicProperties(new AttributesImpl(), this, defaultStyle);
    }

    @Override
    public String getFamily() {
        return "table-cell";
    }

    public TableCellProperties getTableCellProperties() {
        return tableCellProperties;
    }

    @Override
    public ParagraphProperties getParagraphProperties() {
        return paragraphProperties;
    }

    @Override
    public TextProperties getTextProperties() {
        return textProperties;
    }

    @Override
    public LoExtTableCellGraphicProperties getGraphicProperties() {
        return loExtGraphicProperties;
    }

    @Override
    public void writeObject(SerializationHandler output)
        throws SAXException {

        SaxContextHandler.startElement(output, getNamespace(), getLocalName(), getQName());
        writeAttributes(output);
        tableCellProperties.writeObject(output);
        paragraphProperties.writeObject(output);
        textProperties.writeObject(output);
        loExtGraphicProperties.writeObject(output);
        writeMapStyleList(output);
        SaxContextHandler.endElement(output, getNamespace(), getLocalName(), getQName());
    }

    @Override
    public void mergeAttrs(StyleBase styleBase) {
        if(styleBase instanceof IParagraphProperties) {
            getParagraphProperties().mergeAttrs(((IParagraphProperties)styleBase).getParagraphProperties());
        }
        if(styleBase instanceof ITextProperties) {
            getTextProperties().mergeAttrs(((ITextProperties)styleBase).getTextProperties());
        }
        if(styleBase instanceof IGraphicProperties) {
            getGraphicProperties().mergeAttrs(((IGraphicProperties)styleBase).getGraphicProperties());
        }
        if(styleBase instanceof StyleTableCell) {
            getTableCellProperties().mergeAttrs(((StyleTableCell)styleBase).getTableCellProperties());
        }
    }

    @Override
    public void applyAttrs(StyleManager styleManager, JSONObject attrs)
        throws JSONException {

        getParagraphProperties().applyAttrs(styleManager, attrs);
        getTextProperties().applyAttrs(styleManager, attrs);
        getTableCellProperties().applyAttrs(styleManager, attrs);
        getGraphicProperties().applyAttrs(styleManager, attrs);
        if(styleManager.getDocumentType().equals("presentation")) {
            final AttributesImpl cellAttributes = tableCellProperties.getAttributes();
            final AttributesImpl paragraphAttributes = paragraphProperties.getAttributes();
            final AttributeImpl borderTop = cellAttributes.remove("fo:border-top");
            if(borderTop!=null) {
                paragraphAttributes.setValue(borderTop);
            }
            final AttributeImpl borderLeft = cellAttributes.remove("fo:border-left");
            if(borderLeft!=null) {
                paragraphAttributes.setValue(borderLeft);
            }
            final AttributeImpl borderRight = cellAttributes.remove("fo:border-right");
            if(borderRight!=null) {
                paragraphAttributes.setValue(borderRight);
            }
            final AttributeImpl borderBottom = cellAttributes.remove("fo:border-bottom");
            if(borderBottom!=null) {
                paragraphAttributes.setValue(borderBottom);
            }
        }
        final JSONObject cellAttrs = attrs.optJSONObject("cell");
        if(cellAttrs!=null) {
            final String formatCode = cellAttrs.optString("formatCode");
            if(!formatCode.isEmpty()) {
                attributes.setValue(Namespaces.STYLE, "data-style-name", "style:data-style-name",
                    styleManager.applyDataStyle(formatCode, null, cellAttrs.optInt("formatId", -1), false, isAutoStyle(), isContentStyle()));
            }
            else {
                attributes.remove("style:date-style-name");
            }
        }
    }

    @Override
    public void createAttrs(StyleManager styleManager, OpAttrs attrs) {
        Object alignment = null;
        paragraphProperties.createAttrs(styleManager, isContentStyle(), attrs);
        alignment = attrs.remove("paragraph", "alignment");
        textProperties.createAttrs(styleManager, isContentStyle(), attrs);
        if(alignment!=null) {
            attrs.getMap("cell", true).put("alignHor", alignment);
        }
        tableCellProperties.createAttrs(styleManager, isContentStyle(), attrs);
        final String dataStyleName = attributes.getValue("style:data-style-name");
        if(dataStyleName!=null) {
            String formatCode = "";
            final StyleBase dataStyle = styleManager.getStyle(dataStyleName, "data-style", isContentStyle());
            if(dataStyle instanceof NumberStyleBase) {
                formatCode = ((NumberStyleBase)dataStyle).getFormat(styleManager, null, isContentStyle());
            }
            attrs.getMap("cell", true).put("formatCode", formatCode);
        }
        if (styleManager.getDocumentType().equals("presentation")) {
            loExtGraphicProperties.createAttrs(styleManager, isContentStyle(), attrs);
            final Map<String, Object> paragraphAttrs = attrs.getMap("paragraph", false);
            if(paragraphAttrs!=null) {
                final Object borderTop = paragraphAttrs.remove("borderTop");
                if(borderTop!=null) {
                    attrs.getMap("cell", true).put("borderTop", borderTop);
                }
                final Object borderLeft = paragraphAttrs.remove("borderLeft");
                if(borderLeft!=null) {
                    attrs.getMap("cell", true).put("borderLeft", borderLeft);
                }
                final Object borderRight = paragraphAttrs.remove("borderRight");
                if(borderRight!=null) {
                    attrs.getMap("cell", true).put("borderRight", borderRight);
                }
                final Object borderBottom = paragraphAttrs.remove("borderBottom");
                if(borderBottom!=null) {
                    attrs.getMap("cell", true).put("borderBottom", borderBottom);
                }
                final Object borderInside = paragraphAttrs.remove("borderInside");
                if(borderInside!=null) {
                    attrs.getMap("cell", true).put("borderInsideHor", borderInside);
                    attrs.getMap("cell", true).put("borderInsideVert", borderInside);
                }
            }
        }
    }

    @Override
    protected int _hashCode() {

        int result = 0;

        result += tableCellProperties.hashCode();
        result += paragraphProperties.hashCode();
        result += textProperties.hashCode();
        result += loExtGraphicProperties.hashCode();

        return result;
    }

    @Override
    protected boolean _equals(StyleBase e) {
        final StyleTableCell other = (StyleTableCell)e;
        if(!tableCellProperties.equals(other.tableCellProperties)) {
            return false;
        }
        if(!paragraphProperties.equals(other.paragraphProperties)) {
            return false;
        }
        if(!textProperties.equals(other.textProperties)) {
            return false;
        }
        if(!loExtGraphicProperties.equals(other.loExtGraphicProperties)) {
            return false;
        }
        return true;
    }

    @Override
    public StyleTableCell clone() {
        final StyleTableCell clone = (StyleTableCell)_clone();
        clone.tableCellProperties = tableCellProperties.clone();
        clone.paragraphProperties = paragraphProperties.clone();
        clone.textProperties = textProperties.clone();
        clone.loExtGraphicProperties = loExtGraphicProperties.clone();
        return clone;
    }
}

==> open-xchange-documents-backend_7.10.3.orig.tar.bz2: open-xchange-documents-backend-7.10.3/com.openexchange.office.filter.odf/src/com/openexchange/office/filter/odf/styles/StyleTableCell.java <==
/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2016-2020 OX Software GmbH
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

/**
 * @author sven.jacobi@open-xchange.com
 */

package com.openexchange.office.filter.odf.styles;

import java.util.HashMap;
import java.util.Map;
import org.apache.xml.serializer.SerializationHandler;
import org.json.JSONException;
import org.json.JSONObject;
import org.xml.sax.SAXException;
import com.openexchange.office.filter.api.OCKey;
import com.openexchange.office.filter.odf.AttributeImpl;
import com.openexchange.office.filter.odf.AttributesImpl;
import com.openexchange.office.filter.odf.Border;
import com.openexchange.office.filter.odf.Namespaces;
import com.openexchange.office.filter.odf.OpAttrs;
import com.openexchange.office.filter.odf.SaxContextHandler;
import com.openexchange.office.filter.odf.properties.LoExtTableCellGraphicProperties;
import com.openexchange.office.filter.odf.properties.ParagraphProperties;
import com.openexchange.office.filter.odf.properties.TableCellProperties;
import com.openexchange.office.filter.odf.properties.TextProperties;

final public class StyleTableCell extends StyleBase implements ITextProperties, IParagraphProperties, IGraphicProperties {

    private TableCellProperties             tableCellProperties = new TableCellProperties(new AttributesImpl());
    private ParagraphProperties             paragraphProperties = new ParagraphProperties(new AttributesImpl());
    private TextProperties                  textProperties      = new TextProperties(new AttributesImpl());
    private LoExtTableCellGraphicProperties loExtGraphicProperties;

    public StyleTableCell(String name, boolean automaticStyle, boolean contentStyle) {
        super(StyleFamily.TABLE_CELL, name, automaticStyle, contentStyle);

        loExtGraphicProperties = new LoExtTableCellGraphicProperties(new AttributesImpl(), this, false);
    }

    public StyleTableCell(String name, AttributesImpl attributesImpl, boolean defaultStyle, boolean automaticStyle, boolean contentStyle) {
        super(name, attributesImpl, defaultStyle, automaticStyle, contentStyle);

        loExtGraphicProperties = new LoExtTableCellGraphicProperties(new AttributesImpl(), this, defaultStyle);
    }

    @Override
    public StyleFamily getFamily() {
        return StyleFamily.TABLE_CELL;
    }

    public TableCellProperties getTableCellProperties() {
        return tableCellProperties;
    }

    @Override
    public ParagraphProperties getParagraphProperties() {
        return paragraphProperties;
    }

    @Override
    public TextProperties getTextProperties() {
        return textProperties;
    }

    @Override
    public LoExtTableCellGraphicProperties getGraphicProperties() {
        return loExtGraphicProperties;
    }

    @Override
    public void writeObject(SerializationHandler output)
        throws SAXException {

        SaxContextHandler.startElement(output, getNamespace(), getLocalName(), getQName());
        writeAttributes(output);
        tableCellProperties.writeObject(output);
        paragraphProperties.writeObject(output);
        textProperties.writeObject(output);
        loExtGraphicProperties.writeObject(output);
        writeMapStyleList(output);
        SaxContextHandler.endElement(output, getNamespace(), getLocalName(), getQName());
    }

    @Override
    public void mergeAttrs(StyleBase styleBase) {
        if(styleBase instanceof IParagraphProperties) {
            getParagraphProperties().mergeAttrs(((IParagraphProperties)styleBase).getParagraphProperties());
        }
        if(styleBase instanceof ITextProperties) {
            getTextProperties().mergeAttrs(((ITextProperties)styleBase).getTextProperties());
        }
        if(styleBase instanceof IGraphicProperties) {
            getGraphicProperties().mergeAttrs(((IGraphicProperties)styleBase).getGraphicProperties());
        }
        if(styleBase instanceof StyleTableCell) {
            getTableCellProperties().mergeAttrs(((StyleTableCell)styleBase).getTableCellProperties());
        }
    }

    @Override
    public void applyAttrs(StyleManager styleManager, JSONObject attrs)
        throws JSONException {

        final String documentType = styleManager.getDocumentType();
        getParagraphProperties().applyAttrs(styleManager, attrs);
        getTextProperties().applyAttrs(styleManager, attrs);
        if(documentType.equals("text")) {
            // text is sending border attributes via table family
            getTableCellProperties()._applyAttrs(attrs.optJSONObject(OCKey.TABLE.value()));
            getTableCellProperties()._applyAttrs(attrs.optJSONObject(OCKey.CELL.value()));
        }
        else {
            getTableCellProperties().applyAttrs(styleManager, attrs);
        }
        getGraphicProperties().applyAttrs(styleManager, attrs);
        if(documentType.equals("presentation")) {
            final AttributesImpl cellAttributes = tableCellProperties.getAttributes();
            final AttributesImpl paragraphAttributes = paragraphProperties.getAttributes();
            final AttributeImpl borderTop = cellAttributes.remove("fo:border-top");
            if(borderTop!=null) {
                paragraphAttributes.setValue(divideInHalf(borderTop));
            }
            final AttributeImpl borderLeft = cellAttributes.remove("fo:border-left");
            if(borderLeft!=null) {
                paragraphAttributes.setValue(divideInHalf(borderLeft));
            }
            final AttributeImpl borderRight = cellAttributes.remove("fo:border-right");
            if(borderRight!=null) {
                paragraphAttributes.setValue(divideInHalf(borderRight));
            }
            final AttributeImpl borderBottom = cellAttributes.remove("fo:border-bottom");
            if(borderBottom!=null) {
                paragraphAttributes.setValue(divideInHalf(borderBottom));
            }
        }
        final JSONObject cellAttrs = attrs.optJSONObject(OCKey.CELL.value());
        if(cellAttrs!=null) {
            final String formatCode = cellAttrs.optString(OCKey.FORMAT_CODE.value());
            if(!formatCode.isEmpty()) {
                attributes.setValue(Namespaces.STYLE, "data-style-name", "style:data-style-name",
                    styleManager.applyDataStyle(formatCode, null, cellAttrs.optInt(OCKey.FORMAT_ID.value(), -1), false, isAutoStyle(), isContentStyle()));
            }
            else {
                attributes.remove("style:date-style-name");
            }
        }
    }

    private static AttributeImpl divideInHalf(AttributeImpl source) {
        final Border border = new Border();
        border.applyFoBorder(source.getValue());
        final Integer width = border.getWidth();
        if(width!=null) {
            border.setWidth(Double.valueOf(((width+0.5)/2)).intValue());
        }
        return new AttributeImpl(source.getUri(), source.getQName(), source.getLocalName(), border.toString());
    }

    @Override
    public void createAttrs(StyleManager styleManager, OpAttrs attrs) {
        Object alignment = null;
        paragraphProperties.createAttrs(styleManager, isContentStyle(), attrs);
        alignment = attrs.remove(OCKey.PARAGRAPH.value(), OCKey.ALIGNMENT.value());
        textProperties.createAttrs(styleManager, isContentStyle(), attrs);
        if(alignment!=null) {
            attrs.getMap(OCKey.CELL.value(), true).put(OCKey.ALIGN_HOR.value(), alignment);
        }
        tableCellProperties.createAttrs(styleManager, isContentStyle(), attrs);
        final String dataStyleName = attributes.getValue("style:data-style-name");
        if(dataStyleName!=null) {
            String formatCode = "";
            final StyleBase dataStyle = styleManager.getStyle(dataStyleName, StyleFamily.DATA_STYLE, isContentStyle());
            if(dataStyle instanceof NumberStyleBase) {
                formatCode = ((NumberStyleBase)dataStyle).getFormat(styleManager, null, isContentStyle());
            }
            attrs.getMap(OCKey.CELL.value(), true).put(OCKey.FORMAT_CODE.value(), formatCode);
        }
        if (styleManager.getDocumentType().equals("presentation")) {
            loExtGraphicProperties.createAttrs(styleManager, isContentStyle(), attrs);
            final OpAttrs paragraphAttrs = attrs.getMap(OCKey.PARAGRAPH.value(), false);
            if(paragraphAttrs!=null) {
                final Object borderTop = paragraphAttrs.remove(OCKey.BORDER_TOP.value());
                if(borderTop!=null) {
                    attrs.getMap(OCKey.CELL.value(), true).put(OCKey.BORDER_TOP.value(), doubleWidth(borderTop));
                }
                final Object borderLeft = paragraphAttrs.remove(OCKey.BORDER_LEFT.value());
                if(borderLeft!=null) {
                    attrs.getMap(OCKey.CELL.value(), true).put(OCKey.BORDER_LEFT.value(), doubleWidth(borderLeft));
                }
                final Object borderRight = paragraphAttrs.remove(OCKey.BORDER_RIGHT.value());
                if(borderRight!=null) {
                    attrs.getMap(OCKey.CELL.value(), true).put(OCKey.BORDER_RIGHT.value(), doubleWidth(borderRight));
                }
                final Object borderBottom = paragraphAttrs.remove(OCKey.BORDER_BOTTOM.value());
                if(borderBottom!=null) {
                    attrs.getMap(OCKey.CELL.value(), true).put(OCKey.BORDER_BOTTOM.value(), doubleWidth(borderBottom));
                }
                final Object borderInside = paragraphAttrs.remove(OCKey.BORDER_INSIDE.value());
                if(borderInside!=null) {
                    attrs.getMap(OCKey.CELL.value(), true).put(OCKey.BORDER_INSIDE_HOR.value(), doubleWidth(borderInside));
                    attrs.getMap(OCKey.CELL.value(), true).put(OCKey.BORDER_INSIDE_VERT.value(), doubleWidth(borderInside));
                }
            }
        }
    }

    private static Object doubleWidth(Object source)  {
        if(source instanceof Map) {
            final Object width = ((Map)source).get(OCKey.WIDTH.value());
            if(width instanceof Number) {
                final Map<String, Object> dest = new HashMap<String, Object>(((Map) source).size());
                StyleManager.deepCopy((Map)source, dest);
                dest.put(OCKey.WIDTH.value(), ((Number)width).intValue() * 2);
                return dest;
            }
        }
        return source;
    }

    @Override
    protected int _hashCode() {

        int result = 0;

        result += tableCellProperties.hashCode();
        result += paragraphProperties.hashCode();
        result += textProperties.hashCode();
        result += loExtGraphicProperties.hashCode();

        return result;
    }

    @Override
    protected boolean _equals(StyleBase e) {
        final StyleTableCell other = (StyleTableCell)e;
        if(!tableCellProperties.equals(other.tableCellProperties)) {
            return false;
        }
        if(!paragraphProperties.equals(other.paragraphProperties)) {
            return false;
        }
        if(!textProperties.equals(other.textProperties)) {
            return false;
        }
        if(!loExtGraphicProperties.equals(other.loExtGraphicProperties)) {
            return false;
        }
        return true;
    }

    @Override
    public StyleTableCell clone() {
        final StyleTableCell clone = (StyleTableCell)_clone();
        clone.tableCellProperties = tableCellProperties.clone();
        clone.paragraphProperties = paragraphProperties.clone();
        clone.textProperties = textProperties.clone();
        clone.loExtGraphicProperties = loExtGraphicProperties.clone();
        return clone;
    }
}
